{{
  *************************************************
  *                TV_Plot Object                 *
  *        Plots 2 floating point values          *
  *                Version 1.0                    *
  *                  5/26/2007                    *
  *         Primary Author: Martin Hebel          *
  *       Electronic Systems Technologies         *
  *   Southern Illinois University Carbondale     *
  *            www.siu.edu/~isat/est              *
  *                                               *
  * Questions? Please post on the Propeller forum *
  *       http://forums.parallax.com/forums/      *
  * Some code adapted from forums post -          *
  * apologies to that author for not being        *
  * able to find name again.                      *
  *************************************************
  *     --- Distribute Freely Unmodified ---      *
  *************************************************

 Example usage:
  Plot    : "TV_Plot_obj" 

  Plot.start
  Plot.MaxTime(20)            ' Set max time in seconds, approximate value
  Plot.MaxVal(100)            ' Set max Y value
  Plot.freshscreen            ' Clean up plot area

  Plot.PlotA(5.2)             ' plot data - floats
  Plot.PlotB(fp.ffloat(x))
  Plot.UpdateText(chA,ChB)    ' Updates time and values in text area using float value

  Plot.PlotIntA(5)            ' plot data - Integers
  Plot.PlotIntB(X)
  Plot.UpdateIntText(chA,ChB) ' Updates time and values in text area using integers
  
 }}
  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $5000
  display_base = $5000            ' not double-buffer, will have some flicker

  ' Colors
  CL_Black      = $02
  CL_Grey1      = $03
  CL_Grey2      = $04
  CL_Grey3      = $05
  CL_Grey4      = $06
  CL_White      = $07
  CL_Blue       = $0A
  CL_DarkBlue   = $3A
  CL_DarkGreen  = $4A
  CL_Red        = $48
  CL_Brown      = $28
  CL_Yellow     = $9E
  CL_Purple     = $88
  CL_DarkPurple = $EA

VAR   '' Global Variables and Cog Stack Space

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

   word  screen[x_tiles * y_tiles]
   long  colors[2]

   Long     Stack1[100]
   Long     x

   Long TimeStart
   Long Seconds
   Long Count
   Long MaxSeconds
   Long X1Last
   Long Y1Last
   Long X2Last
   Long Y2Last
   Long YMax

obj
  gr    : "graphics"
  TV    : "TV"
  FP    : "FloatMath"
  FS    : "FloatString"

PUB Start
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)
  
  SetColorPalette(0,CL_Yellow,CL_Black,CL_Red,CL_Purple)
  SetColorPalette(1,CL_Black,$F8,CL_Red,CL_Blue)

 
  'init tile screen
  SetAreaColor(0,0,TV_HC-1,TV_VC-1,0) ' Defaults all Screen to Color Palette #0


  gr.start
  gr.setup(16, 12, 0, 0, bitmap_base)

  'repeat
  MaxSeconds := 60  ' default time base (approximate timing only)
  YMax := 100       ' default Ymax value
  freshScreen       ' Plot a fresh screen

Pub DrawGrid| xt, yt
  gr.color(1)
  gr.width(0)
  Repeat Xt From 31 to 255 step 22      ' draw vertical lines
      gr.Plot(Xt,0)
      gr.line(Xt,150)  
  Repeat Yt From 0 to 150 step 30       ' draw horizontal lines
      gr.Plot(31,Yt)
      gr.line(251,Yt)


Pub SetAreaColor(X1,Y1,X2,Y2,ColorIndex)|DX,DY
  Repeat DX from X1 to X2
    Repeat DY from Y1 to Y2
      SetTileColor(DX,DY,ColorIndex)
    
Pub SetTileColor( xt, yt, ColorIndex)
   screen[yt * tv_hc + xt] := display_base >> 6 + yt + xt * tv_vc + ((ColorIndex & $3F) << 10)
 
Pub SetColorPalette(ColorIndex,Color1,Color2,Color3,Color4)
  colors[ColorIndex] := (Color1) + (Color2 << 8) +  (Color3 << 16) + (Color4 << 24)  

Pub FreshScreen | i,j
  gr.clear
  DrawGrid
  gr.color(1)
  gr.width(0)
  PlaceText
  gr.textmode(1,1,6,%1001)                   ' Right just, center top-bottom justify

  repeat i from 1 to 5                       ' label vertical
    j := fp.ffloat(YMax * i / 5)
    gr.text(0,0,string(" "))
    gr.text(29,i * 30,fs.floattostring(j))
    
  gr.textmode(1,1,6,%1000)                   ' right justify
  repeat i from 1 to 5                       ' label horizontal
     j := fp.ffloat(maxseconds * i/5)
     gr.text(32+(i*220/5),155,fs.floattostring(j))
     
  gr.textmode(1,1,6,%0000)
  TimeStart := cnt                           ' Start timer count
  Count := 0
  X1Last :=0
  Y1Last :=0
  X2Last :=0
  Y2Last :=0

Pub PlaceText
  gr.color(2)                                
  gr.textmode(1,1,6,%0000)   
  gr.text(0,177,string("Plot A:"))
  gr.color(3)
  gr.text(95,177,string("Plot B:"))
  gr.color(1)
  gr.text(180,177,string("Time:"))

Pub UpdateIntText(ChA, ChB)
   UpdateText(fp.ffloat(ChA),fp.ffloat(ChB))

Pub UpdateText (ChA,ChB)| Time
  fs.setprecision(3)
  gr.width(0)
  gr.color(0)
  gr.box(0,170,256,40)
  PlaceText
  gr.text(0,0,string(" "))                    ' things to get around numeric quircks on screen
  gr.color(1) 
  gr.text(140,177,FS.FloatToString(ChB))      ' display channel B
  gr.text(0,0,string(" "))
  gr.text(45,177,FS.FloatToString(ChA))       ' display channel A
  Time := fp.fdiv(FP.FFloat(timer),10.0)
  gr.text(0,0,string(" "))
  gr.text(215,177,fs.floattostring(Time)) 

Pub Timer                                     ' calculate time
  If maxseconds < 26
     return (cnt - timestart) / (clkfreq /10)
  else
    Count := Count + ((cnt - timestart)/(clkfreq/100))
    timestart := cnt
    return Count / 10
    

Pub MaxTime(val)                             ' set max seconds
    MaxSeconds := val

Pub MaxVal(val)                              ' set max Y value
    YMax := Val
    
Pub PlotIntA(value)
    PlotA(fp.ffloat(value))
    
Pub PlotA (value) | XP, YP
    if timer > maxseconds * 10
       freshscreen 
    gr.width(2)
    YP := fp.fRound(fp.fdiv(fp.fmul(value,150),ymax)) <# 150
    XP := timer * 22 / maxSeconds + 32  
    gr.color(2)
    gr.plot(XP,YP)
    If (X1Last == 0) and (Y1Last == 0)
      X1Last := XP
      Y1Last := YP 
    gr.line(X1Last,Y1Last)
    X1Last := XP
    Y1Last := YP    

Pub PlotIntB(value)
    PlotB(fp.ffloat(value))
    
Pub PlotB (value) | XP, YP
    if timer > maxseconds * 10
       freshscreen 
    gr.width(2)
    YP := fp.fRound(fp.fdiv(fp.fmul(value,150),ymax)) <# 150
    XP := timer * 22 / maxSeconds + 30  
    gr.color(3)
    gr.plot(XP,YP)
    If (X2Last == 0) and (Y2Last == 0)
      X2Last := XP
      Y2Last := YP 
    gr.line(X2Last,Y2Last)
    X2Last := XP
    Y2Last := YP   

                                                                                         

               
DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000      'broadcast
                        long    0               'auralcog